package com.cht.iot.conv;

import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;

/* 
 * Modbus Adaptor 支援 Modbus/RTU over TCP
 * 機密性資料如: id(+register:m001;)、port(/dev/ttyUSB0)、host(211.20.181.199)僅為說明用途之假資料
 * 請您遵循下列步驟，並根據您的專案資料及設備進行修改，方可運行
 */

public class Converter {
	static final Logger LOG = LoggerFactory.getLogger(Converter.class);
	
	public static void main(String[] args) throws Exception {
		
		// 1. 請填寫設備正確的Connection port
		String port = "/dev/ttyUSB0";	
		
		// 2. 請填寫IoT大平台host name
		String host = "211.20.181.199"; 
		
		// 3. 請填寫IoT大平台Modbus設備URI，sn://m001對應+register:m001。
		// 每個設備的 URI 不能重複，請利用 IoT 平台自動產生的數值即可，不要自行變更。
		String id = "+register:m001;";
		
		// 4. 請填寫設備正確的BaudRate
		int baudRate = 19200;
		
		// 5. 請根據設備回應時間彈性調整
		int connectionTimeout = 30;
		int receiveTimeout = 70;
					
		for (;;) {
			try {
				SerialPort sp = (SerialPort) CommPortIdentifier.getPortIdentifier(port).open(port, connectionTimeout);
				sp.setSerialPortParams(baudRate, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
				sp.enableReceiveTimeout(receiveTimeout);
						
				try {
					
					InputStream sis = sp.getInputStream();
					OutputStream sos = sp.getOutputStream();
					
					
					Socket socket = new Socket(host, 10500);
					try {
						OutputStream os = socket.getOutputStream();
						InputStream is = socket.getInputStream();
						
						os.write(id.getBytes());
						
						Fetcher f = new Fetcher("fromSerial", sis, os);						
						f.start();					
						
						try {
							Fetcher.fetch("fromServer", is, sos, 10L, new Fetcher.NullListener());
							
						} finally {
							f.close();
						}
						
					} finally {
						socket.close();
					}
					
				} finally {
					sp.close();
				}
				
			} catch (Exception e) {
				LOG.error("Error", e);
			}
			
			LOG.warn("Disconnect from server");
			
			Thread.sleep(3000);
		}		
	}
}
